
namespace Terminal
{
	public class Menu : Gtk.Menu
	{
		Vte.Terminal terminal;
		Pid pid;

		public Menu ()
		{
			terminal = new Vte.Terminal ();
			terminal.fork_command_full (Vte.PtyFlags.DEFAULT, "~", { Vte.get_user_shell () }, null, SpawnFlags.SEARCH_PATH, null, out pid);
			terminal.width_request = 480;
			terminal.height_request = 320;
			var item = new Gtk.MenuItem ();
			item.add (terminal);
			add (item);
			show_all ();

			show.connect (() => {
				Idle.add (() => {
					terminal.grab_focus ();
					return false;
				});
			});
		}

		public override bool button_press_event (Gdk.EventButton event)
		{
			return terminal.button_press_event (event);
		}

		public override bool button_release_event (Gdk.EventButton event)
		{
			return terminal.button_release_event (event);
		}

		public override bool key_press_event (Gdk.EventKey event)
		{
			return terminal.key_press_event (event);
		}

		public override bool key_release_event (Gdk.EventKey event)
		{
			return terminal.key_release_event (event);
		}
	}

	public class Main : Object
	{
		public Menu menu { get; private set; }

		string? current_shortcut;
		Cancellable? current_search = null;

		public Main ()
		{
			menu = new Menu ();

			// shortcut
			Keybinder.init ();
			update_shortcut ();
		}

		public void update_shortcut ()
		{
			current_shortcut = "<Super><Shift><Alt>space";
			Keybinder.bind (current_shortcut, handle_shortcut, this);
		}

		public void show_menu ()
		{
			(menu.get_attach_widget () as Gtk.MenuItem).activate_item ();
			menu.deselect ();
		}
	}

	public static Gdk.Pixbuf? find_icon (string name, int size)
	{
		try {
			var icon = Icon.new_for_string (name);
			if (icon == null)
				return null;
			var info = Gtk.IconTheme.get_default ().lookup_by_gicon (icon, size, Gtk.IconLookupFlags.FORCE_SIZE);
			if (info == null)
				return null;
			return info.load_icon ();
		} catch (Error e) { warning (e.message); }

		return null;
	}

	public static void handle_shortcut (string key, void* data)
	{
		var self = (Main)data;
		// unfortunately things are not that easy here. Gtk throws an error about no device
		// when trying to grab. Waiting until the key is released solves this problem, so
		// we keep checking if we see something already
		Idle.add (() => {
			if (!self.menu.visible)
				self.show_menu ();
			return !self.menu.visible;
		});
		self.show_menu ();
	}
}

